----------------------------------------------------------------------------
-- Top level: top_K233_point_multiplication.vhd
-- Point multiplication in K163 (K163_point_multiplication.vhd)
--
-- Introduces the point coordinates and k through the same port
--
----------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity top_K233_point_multiplication is
port (
  inData: in std_logic_vector(232 downto 0);  
  xP_data, yP_data, k_data: in std_logic;
  clk, reset, start: in std_logic;
  outData: inout std_logic_vector(232 downto 0);
  xQ_or_yQ: in std_logic;
  done: out std_logic
  );
end top_K233_point_multiplication;

architecture circuit of top_K233_point_multiplication is

  component K233_point_multiplication is
  port (
    xP, yP, k: in std_logic_vector(232 downto 0);
    clk, reset, start: in std_logic;
    xQ, yQ: inout std_logic_vector(232 downto 0);
    done: out std_logic
    );
  end component K233_point_multiplication;

  signal xP, yP, k, xQ, yQ: std_logic_vector (232 downto 0);

begin

  theComp: K233_point_multiplication port map(
                    xP => xP, yP => yP, k => k,
                    clk => clk, reset => reset, start => start,
                    xQ => xQ, yQ => yQ, done => done );

  registers: process(clk)
  begin
  if clk' event and clk = '1' then 
    if xP_data = '1' then xP <= inData; end if;
    if yP_data = '1' then yP <= inData; end if;
    if k_data = '1'  then k <= inData; end if;
  end if;
  end process;

  outData <= xQ when xQ_or_yQ = '0' else yQ;

end circuit;

